!> \file 3D-physique-gen_mod.f90
!! Module de declaration des variables generales
!! de la physique du modele.
!<

!> \namespace module3D_phy
!! Module de declaration des variables generales
!! de la physique du modele.
!! \author CatRitz
!! \date 30 juin 2007
!! @note Used module
!! @note   - use geography
!! @note   - use runparam
!<

module MODULE3D_phy

  ! declaration des variables
  ! implicit none
  use geography, only: nx,ny,nz,nzm   ! indirectement donne acces au nx,ny, appropries

  implicit none
  real, parameter :: PI = 3.1415927


  !   **** integer declaration  *************************************

  integer ::  ispinup                   !< pour les differentes etapes du spinup
  integer ::  err                       !< pour l'allocation des tableaux
  integer ::  igrdline                  !< si 1 fixe la position en jouant sur la fusion shelf
  integer ::  ibmelt_inv                !< si 1 inversion du bmelt (avec igrdline=1)
  integer ::  i_resolmeca               !< defini le type d'association SIA-L1
  integer ::  inv_beta                  !< 0 : run std, 1 : inversion du beta
  integer ::  ICOUPLE                   !< 
  integer ::  ICOMPTEUR                 !< reprise fichier cptr 
  integer ::  IOUT                      !< type de fichier de sortie pour reprise
  integer ::  iglen                     !< compteur pour la boucle flowlaw
  integer ::  ISYNCHRO                  !< synchronisation pas de temps dt et dtt 
  integer ::  Schoof                    !< 0 : pas de flux de Schoof , 1 : flux de Schoof a la grounding line
  integer ::  NTMAX                     !< 

  integer ::  ITEMP                     !< 
  integer ::  niter_nolin               !< number of velocity iterations for non-linear dragging laws
  integer ::  ndebug                    !< pour faire des sorties de debug a plusieurs pas de temps
  integer ::  ndebug_max                !< pour faire des sorties de debug a plusieurs pas de temps

  !     ************ common des scalaires *************************

  real :: coefbmshelf                   !< coef permettant de faire varier la fusion basale :

  real ::  DT                           !< pas de temps court
  real ::  DTT                          !< pas de temps long
  real ::  DTMIN                        !< pas de temps mini
  real ::  DX1                          !< 1/dx
  real ::  DTDX                         !< dt/dx
  real ::  dtdx2                        !< dt/dx**2
  real ::  DTMAX                        !< 
  real ::  SECYEAR                      !< for relation an/seconds
  real ::  sealevel                     !< niveau des mers, afq: now used as eustatic sea level
  real (kind=kind(0.d0)) ::  TIME       !< temps : en double precision
  real ::  TAFOR                        !< temperature forcing : annual
  real ::  TESTDIAG                     !< for time step calculation (icethick)
  real ::  V_limit                      !< vitesse maxi (limitateur de flux pour conserv masse)
  real ::  betamax                      !< (Pa) frottement maxi sous les streams


  !     ************** common des caracteres **************
  !character(len=8), dimension(NPL) :: VARNAME  ! 
  character(len=320) :: reprcptr                ! nom du fichier de reprise CPTR

  !     *************** common des tableaux a 1 dimension *****
  real,dimension(nz) :: e               !< vertical coordinate in ice, scaled to H zeta
  real,dimension(nz) :: CDE             !<  

  !    ************* common des tableaux a deux dimensions ******

  integer,dimension(nx,ny) :: FRONT     !< point d'un shelf situe sur le front
  integer,dimension(nx,ny) :: gr_line_schoof ! points ou on impose le flux de schoof (pour sorties)
  integer,dimension(nx,ny) :: gr_line   !< points grounding line pour les sorties
  integer,dimension(nx,ny) :: ICE       !< presence de glace au point considere,
  !< seuil 0 si pose, 1 si flottant
  integer,dimension(nx,ny) :: MASQUE_BMSHELF !< masque pour determiner a quel bassin oceanique 
  !< appartiennent les points de la grille
  integer,dimension(nx,ny) :: MK0          !< masks (ice sheet, max, above water, below water, 1)
  integer,dimension(nx,ny) :: Mk_init      !< initial mask (with islands, outcrops, ...
  integer,dimension(nx,ny) :: MK           !< masks (ice sheet, max, above water, below water, 1)
  integer,dimension(nx,ny) :: IBASE     !< type de base (froide, temperee)

  real,dimension(nx,ny) :: ACC          !< Accumulation  'o'
  real,dimension(nx,ny) :: ABL          !< Ablation  'o'  negative quand perte
  real,dimension(nx,ny) :: ABLBORD      !< Ablation dans les zones non englacees
  real,dimension(nx,ny) :: ablbord_dtt  !< Ablation sur le bord de la calotte (flux de glace infrieur  ablation sur le bord) sur dtt
  double precision,dimension(nx,ny) :: B            !< Altitude de la base de la glace  'o'
  real,dimension(nx,ny) :: BDOT         !< derivee de B / t
  double precision,dimension(nx,ny) :: BSOC0        !< ice free bedrock
  real,dimension(nx,ny) :: betamx       !< coefficient beta du frottement basal, betamx=cf*neffmx et tobmx=betamx*uxbar
  real,dimension(nx,ny) :: betamy       !< coefficient beta du frottement basal, betamy=cf*neffmy et tobmy=betamy*uybar
  real, dimension(nx,ny) :: beta_centre !< beta on major node (average)
  real, dimension(nx,ny) :: betamax_2d  !< (Pa) frottement maxi sous les streams
  !
  real,dimension(nx,ny) :: BM           !< mass balance   'o'
  real,dimension(nx,ny) :: BMELT        !< basal melting  'o'
  real,dimension(nx,ny) :: CORRBMELT    !< correction du basal melting  'o'
  real,dimension(nx,ny) :: HWATER       !< hauteur d'eau basale dans le sediment
  real,dimension(nx,ny) :: COEFMXBMELT  !< 
  real,dimension(nx,ny) :: COEFMYBMELT  !< 
  real,dimension(nx,ny) :: CALV         !< calving
  
  real,dimension(nx,ny) :: DDBX         !< UBX/(-SDX) '>'
  real,dimension(nx,ny) :: DDBY         !< UBY/(-SDY) '^'
  real,dimension(nx,ny) :: Diffmx       !< partie diffusive en x
  real,dimension(nx,ny) :: Diffmy       !< partie diffusive en y

  real,dimension(nx,ny) :: DHDT         !< derive lagrangienne de l'epaisseur 'o'
  real,dimension(nx,ny) :: epsxx     !< vitesse de deformation selon x
  real,dimension(nx,ny) :: epsyy     !< vitesse de deformation selon y
  real,dimension(nx,ny) :: epsxy     !< vitesse de deformation selon xy
  real,dimension(nx,ny) :: eps       !< vitesse de deformation (invariant)

  real,dimension(nx,ny) :: FT           !< proportion of year with air temp below 2C
  real,dimension(nx,ny) :: FROTMX       !< beta coefficient used in the remplimat routine
  real,dimension(nx,ny) :: FROTMY       !< 
  real,dimension(nx,ny) :: GHF          !< geothermal heat flux J/m2/a 'o'
  double precision,dimension(nx,ny) :: H            !< ice thickness  'o'
  double precision,dimension(nx,ny) :: H0           !< initial ice thickness, must be initialized before init_iso

! dimensionnements qui etaient avant dans H_prescribe
  real,dimension(nx,ny)    :: Hp           !< H value if prescribed
  real,dimension(nx,ny)    :: Hp0          !< H value if prescribed (reference value)
  integer,dimension(nx,ny) :: i_Hp         !< 1 if H is prescribed on this node, else 0
  integer,dimension(nx,ny) :: i_Hp0        !< i_hp mask reference value does not change with time
  integer, dimension(nx,ny) :: imx_diag    !< masque pour eq elliptique  
  integer, dimension(nx,ny) :: imy_diag    !< masque pour eq elliptique 

  double precision,dimension(nx,ny) :: HDOT         !< ice thickness derivee / t
  real,dimension(nx,ny) :: HDOTWATER
  real,dimension(nx,ny) :: HMX          !< ice thickness moy selon x '>'
  real,dimension(nx,ny) :: HMY          !< ice thickness moy selon y '^'

  real,dimension(nx,ny) :: PHID         !< flux de chaleur lie a la deformation et glissement basal
  real,dimension(nx,ny) :: NEFFMX       !< pression effective '>'
  real,dimension(nx,ny) :: NEFFMY       !< pression effective '^'
  real,dimension(nx,ny) :: TOBMX        !< cisaillement basal '>'
  real,dimension(nx,ny) :: TOBMY        !< cisaillement basal '^'
  double precision,dimension(nx,ny) :: S            !< altitude of ice sheet surface
  real,dimension(nx,ny) :: sealevel_2d  !< local sea surface elevation
  real,dimension(nx,ny) :: SLOPE        !< slope 'o'
  real,dimension(nx,ny) :: SDX          !< slope derivee / x '>'
  real,dimension(nx,ny) :: SDY          !< slope derivee / y '^'
  real,dimension(nx,ny) :: SLOPE2mx     !< = Sdx**2 + Sdymx**2 '>'
  real,dimension(nx,ny) :: SLOPE2my     !< = Sdy**2 + Sdxmy**2 '^'
  double precision,dimension(nx,ny) :: S0           !< altitude actuelle de la surface
!afq -- replaced by sealevel_2D:  real,dimension(nx,ny) :: slv          !< niveau de flottaison (sealevel et lakes)
  real,dimension(nx,ny) :: TJULY        !< Ground air temperature July
  real,dimension(nx,ny) :: TANN         !< Ground air temperature annual
  real,dimension(nx,ny,12) :: Tmois        !< Ground air temperature monthly
  real,dimension(nx,ny) :: TAUB         !< basal shear stress (for output)
  real,dimension(nx,ny) :: TAUSHELF     !< effective stress in ice shelves (vertical av.) 
  real,dimension(nx,ny) :: TS           !< surface ice temperature  'o'
  real,dimension(nx,ny) :: TB           !< basal ice temperature  'o'
  real,dimension(nx,ny) :: TBDOT        !< variation in time of basal temperature 
  real,dimension(nx,ny) :: UZK          !< vertical velocity at the ice surface (kinematic boundary)
  real,dimension(nx,ny) :: UXBAR        !< vertically integrated velocity '>'
  real,dimension(nx,ny) :: UYBAR        !< vertically integrated velocity '^'
  real,dimension(nx,ny) :: uxdef        !< partie due a la deformation
  real,dimension(nx,ny) :: uydef        !< partie due a la deformation
  real,dimension(nx,ny) :: uxflgz       !< vitesse stream-shelf
  real,dimension(nx,ny) :: uyflgz       !< vitesse stream-shelf
  real,dimension(nx,ny) :: UBX          !< basal sliding '>'
  real,dimension(nx,ny) :: UBY          !< basal sliding '^'
  real,dimension(nx,ny) :: UZSDOT       !< variation in time of surface vertical velocity 
  real,dimension(nx,ny) :: XLONG        !< longitude
  real,dimension(nx,ny) :: YLAT         !< latitude
  real,dimension(nx,ny) :: xcc          ! grille Xkm
  real,dimension(nx,ny) :: ycc          ! grille Ykm
  real,dimension(nx,ny) :: BSOC         !< altitude (ou bathymetrie) du socle 'o'
  real,dimension(nx,ny) :: PVI          !< viscosite ice shelf
  real,dimension(nx,ny) :: PVM          !< viscosite ice shelf
  real,dimension(nx,ny) :: Abar         !< coefficient de Glen integre


  ! nouveaux tableaux (aout 2006) pour dragging-sliding
  ! ces tableaux peuvent avoir des roles differents selon les modules de dragging
  ! et de sliding utilises. Ils servent a faire passer des contraintes geographiques entre
  ! le main et les rounines de sliding-dragging.

  integer,dimension(nx,ny) :: mstream_mx  !< masque stream selon x
  integer,dimension(nx,ny) :: mstream_my  !< masque stream selon y
  integer,dimension(nx,ny) :: mstream     !< masque stream sur les noeuds majeurs
  real,dimension(nx,ny) :: drag_mx        !< coefficient pour tenir compte de variations        
  real,dimension(nx,ny) :: drag_my        !< geographiques eventuelles dans le basal drag

  !      ********** common des tableaux a 3 dimensions *****

  real,dimension(nx,ny,nz) :: SUX       !< 
  real,dimension(nx,ny,nz) :: SUY       !< 
  real,dimension(nx,ny,nz) :: TPMP      !< pressure melting point temperature in ice sheet 'o'
  real,dimension(nx,ny,nz) :: UX        !< ice velocity '>'
  real,dimension(nx,ny,nz) :: UY        !< ice velocity '^'
  real,dimension(nx,ny,nz) :: UZR       !< ice velocity and derivatives
  real,dimension(nx,ny,nz) :: ETA       !< 


  ! tableaux qui permettent de faire facilement des sorties debug via ncdf

  real,dimension(nx,ny) :: debug_2D
  real,dimension(nx,ny,220) :: debug_3D

  !  tableaux qui permettent de faire des sorties netcdf avec les variables qui ne sont pas globales
  !  real,dimension(nx,ny,199) :: netcdf_glob           ! pas l'air utilise


  !real,dimension(NX,NY,NPL) :: XXX      !< work array, for temperature and output

  real,dimension(nx,ny,nz+nzm) :: T     !< temperature in ice and mantle 'o'




  !======================= common pour l'equilibrage en temperature
  !real,dimension(NX,NY) :: Uxbilan      ! vitesse de bilan '>'
  !real,dimension(NX,NY) :: Uybilan      ! vitesse de bilan '^'



  ! ===================== Booleens ========================================
  logical,dimension(nx,ny) :: FLOT      !< vrai si flottant (test d'archimede) 'o'
  logical,dimension(nx,ny) :: FLOTMX    !< if flotting (one neighboord  floats) '>'
  logical,dimension(nx,ny) :: FLOTMY    !< if flotting (one neighboord  floats) '^'  
  logical,dimension(nx,ny) :: FRONT_RESC!< comme front (mais boolean) pour remplimat rescue
  logical,dimension(nx,ny) :: FBM       !< 
  logical,dimension(nx,ny) :: GZMX      !< point stream ">"
  logical,dimension(nx,ny) :: GZMY      !< point stream "^"
  logical,dimension(nx,ny) :: FLGZMX    !< points stream ou shelf ou ile (equ. ellpt) ">"
  logical,dimension(nx,ny) :: FLGZMY    !< points stream ou shelf ou ile (equ ellipt) "^"
  logical,dimension(nx,ny) :: ILEMX     !< points ile ">"
  logical,dimension(nx,ny) :: ILEMY     !< points ile "^"
  logical,dimension(nx,ny) :: fleuvemx  !< actual grounded stream
  logical,dimension(nx,ny) :: fleuvemy  !< actual grounded stream
  logical,dimension(nx,ny) :: flot_marais  !< afq -- vrai si flottant et coince entre points poses 'o'
  logical,dimension(nx,ny) :: iceberg   !< point iceberg

  ! ===================== File id =========================================
  integer :: num_param     = 10   !< Id of param_list.dat
  integer :: num_coupe     = 11   !< Id of coupe.ijz
  integer :: num_vitbil    = 12   !< Id of vitbil
  integer :: num_newton    = 15   !< Id of corrextion-newton.res
  integer :: num_file1     = 18   !< Id of 
  integer :: num_file2     = 19   !< Id of 
  integer :: num_forc      = 20   !< Id of forcage
  integer :: num_dat       = 21   !< Id of dat
  integer :: num_cptr      = 30   !< Id of cptr
  integer :: num_debug     = 33   !< Id of ritz debug
  integer :: num_ritz      = 35   !< Id of ritz
  integer :: num_templlib  = 36   !< Id of ritz templlib
  integer :: num_rep_42    = 42   !< Id of reponse
  integer :: num_trace     = 81   !< Id of trace-reboot
  integer :: num_file      = 88   !< Id of 
  integer :: num_ts_ritz   = 89
  integer :: num_kelvin    = 90   !< Id of kelvin.res
  integer :: num_phil      = 91
  integer :: num_templin   = 98   !< Id of templin.res
  integer :: num_sealevel  = 110
  integer :: num_mk_vitbil = 122  !< Id of masque_vitbil
  integer :: num_ic_vo     = 171  !< vostok
  integer :: num_ic_by     = 172  !< byrd
  integer :: num_ic_dm     = 173  !< dml
  integer :: num_ic_dc     = 174  !< dome C
  integer :: num_ic_df     = 175  !< dome Fuji
  integer :: num_file3     = 993
  integer :: num_file4     = 995  !< Id of 
  integer :: num_coor      = 2004 !< Id of coord-Ant-40km

  ! Variables communes au main et aux subroutines du step
  real  :: timemax

end module MODULE3D_PHY
